from typing import Dict, List, Optional, Mapping, Any

from fast_tradier.models.trading.TOSTradierConverter import TOSTradierConverter
from fast_tradier.models.ModelBase import ModelBase
from fast_tradier.models.trading.Sides import OptionOrderSide
from fast_tradier.models.trading.PriceTypes import OptionPriceType
from fast_tradier.models.trading.Duration import Duration
from fast_tradier.models.trading.OrderBase import OrderBase
from fast_tradier.models.ModelBase import NewModelBase
from pydantic import BaseModel, ConfigDict, Field

class OptionLeg(NewModelBase):
    underlying_symbol: Optional[str] = None
    option_symbol: str
    quantity: int
    side: OptionOrderSide
    model_config = ConfigDict(use_enum_values=True, populate_by_name=True)
    
    def reverse_side(self) -> None:
        if self.side == OptionOrderSide.SellToOpen or self.side == OptionOrderSide.SellToOpen.value:
            setattr(self, "side", OptionOrderSide.BuyToClose)
        elif self.side == OptionOrderSide.BuyToOpen or self.side == OptionOrderSide.BuyToOpen.value:
            setattr(self, "side", OptionOrderSide.SellToClose)

class OptionOrder(NewModelBase):
    symbol: str
    price: Optional[float] = None
    stop: Optional[float] = None
    price_type: OptionPriceType = Field(..., alias="type")
    duration: Duration
    option_legs: List[OptionLeg]
    side: Optional[OptionOrderSide] = None # required for single leg option order
    status: Optional[str] = "pending"
    order_class: str = Field("multileg", alias="class") # could be "option"
    id: Optional[int] = None
    tag: Optional[str] = None
    model_config = ConfigDict(use_enum_values=True, populate_by_name=True)

    @property
    def ticker(self) -> str:
        return self.symbol

    def set_price(self, new_price: float) -> None:
        setattr(self, "price", new_price)

    def set_status(self, new_staus: str) -> None:
        setattr(self, "status", new_staus)

    def set_ticker(self, ticker_name: str) -> None:
        setattr(self, "ticker", ticker_name)
    
    def set_id(self, new_id: int) -> None:
        setattr(self, "id", new_id)

    def clone_option_legs(self, reverse_side: bool = False) -> List[OptionLeg]:
        '''deep clone option_legs'''
        cloned_legs = []
        for opt_leg in self.option_legs:
            leg = OptionLeg(**(opt_leg.to_json()))
            if reverse_side:
                leg.reverse_side()

            cloned_legs.append(leg)
        return cloned_legs
    
    def to_json(self) -> Mapping[str, Any]:
        """
        Override to_json so that the format complies to Tradier API.
        Returns:
            Mapping[str, Any]: _description_
        """
        temp_result = self.model_dump(by_alias=True)
        result: Dict[str, Any] = {}
        for k, v in temp_result.items():
            result[k] = v

        if len(self.option_legs) == 1:
            result["option_symbol"] = self.option_legs[0].option_symbol
            result["side"] = self.option_legs[0].side
            result["quantity"] = self.option_legs[0].quantity
        elif len(self.option_legs) > 1:
            for i in range(len(self.option_legs)):
                opt_item = self.option_legs[i]
                symbol_key = f"option_symbol[{i}]"
                result[symbol_key] = opt_item.option_symbol
                side_key = f"side[{i}]"
                result[side_key] = f"{opt_item.side}"
                quant_key = f"quantity[{i}]"
                result[quant_key] = f"{opt_item.quantity}"

        return result